import operator

# Operators used in binary expressions.
OP_AND = 'and'
OP_OR = 'or'

OP_EQ = '=='
OP_NE = '!='
OP_LT = '<'
OP_LTE = '<='
OP_GT = '>'
OP_GTE = '>='
# OP_IN = 'in'

opfunc = {
    OP_AND: operator.__and__,
    OP_OR:  operator.__or__,
    OP_EQ:  operator.eq,
    OP_NE:  operator.ne,
    OP_LT:  operator.lt,
    OP_LTE: operator.le,
    OP_GT:  operator.gt,
    OP_GTE: operator.ge
}

class Node(object):
    """Base-class for any part of a query which shall be composable."""
    def __init__(self, name=None):
        self.name = name

    def _e(op, inv=False):
        """
        Lightweight factory which returns a method that builds an Expression
        consisting of the left-hand and right-hand operands, using `op`.
        """
        def inner(self, rhs):
            if inv:
                return Expression(rhs, op, self)
            return Expression(self, op, rhs)
        return inner

    __and__ = _e(OP_AND)
    __or__ = _e(OP_OR)

    def __eq__(self, rhs):
        return Expression(self, OP_EQ, rhs)
    def __ne__(self, rhs):
        return Expression(self, OP_NE, rhs)

    __lt__ = _e(OP_LT)
    __le__ = _e(OP_LTE)
    __gt__ = _e(OP_GT)
    __ge__ = _e(OP_GTE)
    # __lshift__ = _e(OP_IN)

    def __str__(self):
        if self.name: return self.name
        else: return repr(self)

class Expression(Node):
    """A binary expression, e.g `foo == 1` or `bar < 7`"""
    def __init__(self, lhs, op, rhs, flat=False):
        super(Expression, self).__init__()
        self.lhs = lhs
        self.op = op
        self.rhs = rhs
        self.flat = flat

    def __str__(self):
        return '(' + str(self.lhs) + ' ' + str(self.op) + ' ' + str(self.rhs) + ')'

    def eval(self, target):
        if self.op == OP_AND or self.op == OP_OR:
            return opfunc[self.op](self.lhs.eval(target), self.rhs.eval(target))
        else:
            if target.has_key(self.lhs.name):
                return opfunc[self.op](target[self.lhs.name], self.rhs)
            else:
                return False




# -*- coding: utf-8 -*-

import sys
import math

L = 36

class ProgressBar(object):
    def __init__(self, num_iter):
        self.unit = num_iter * 1.0 / L
        self.iter = num_iter
        sys.stderr.write('\r' + self._bar(0) + '\t' + self._percent(0))

    def update(self, val):
        if val >= self.iter-1:
            sys.stderr.write('\r' + self._bar(val) + '\t' + self._percent(val) + '\n')
        else:
            sys.stderr.write('\r' + self._bar(val) + '\t' + self._percent(val))

    def _bar(self, val):
        progress = int( (val+1) / self.unit )
        return '[' + '#' * progress + ' ' * (L-progress) + ']'

    def _percent(self, val):
        return '%.1f' % ( (val+1) * 100.0 / self.iter ) + '%'

if __name__ == '__main__':
    import time
    p = ProgressBar(num_iter=10)
    for i in range(10):
        time.sleep(1)
        p.update(i)

    print 'finish!'



from gq import Selector, mtable_to_dict
import pytest

class TestUtil(object):

    def test_mtable_to_dict(self, mtable, paramdict):
        assert mtable_to_dict(mtable) == paramdict

class Testgq(object):

    def test_type(self, _):
        assert isinstance(_, Selector)

    def test_params(self, _, paramdict):
        assert _.params == paramdict

    def test_val_missing(self, _):
        assert str(_._hoge) == 'hoge'
        assert str(_._fuga) == 'fuga'

if __name__ == '__main__':
    pytest.main()



# -*- coding: utf-8 -*-

from missing import ValMissing
from expr import Node, Expression
from pbar import ProgressBar

import re
import json
import os

def gq(rootdir, params):
    return Selector(rootdir, params)

def mtable_to_dict(maf_id_file):
    """ maf_id_tableからdictへの変換
    """

    out = []

    pattern = re.compile(r'(?P<id>\d+)\t\{(?P<json>.+)\}')
    for line in maf_id_file:
        m = pattern.match(line)
        text = '{"id": ' + m.group('id') + ', ' + m.group('json').replace("'", '"') + '}'
        out.append( json.loads(text) )

    return out

id_pattern = re.compile(r'(?P<id>\d+?)-.+')

def get_id(fname):
    m = id_pattern.match(fname)
    return int(m.group('id'))

class Selector(ValMissing):
    def __init__(self, rootdir, params, curdir=None, filter=None, next_is_or=False, onGet=lambda f: f):
        self.rootdir = os.path.abspath(os.path.expanduser(rootdir))
        self.params = params
        if curdir:
            self.curdir = curdir
        else:
            self.curdir = self.rootdir
        self.filter = filter
        self.next_is_or = next_is_or
        self.onGet = onGet

    def val_missing(self, name):
        if name.startswith('_'):
            return Node(name[1:])
        else:
            raise AttributeError("%r object has no attribute %r" %
                                 (self.__class__, name))

    def __call__(self, d):
        return Selector(self.rootdir, self.params,
                        curdir=os.path.join(self.curdir, d),
                        filter=self.filter,
                        onGet=self.onGet)

    def where(self, expr):
        if not isinstance(expr, Expression):
            raise TypeError("Expression is required")
        elif self.filter:
            if self.next_is_or:
                return Selector(self.rootdir, self.params,
                                curdir=self.curdir,
                                filter=self.filter | expr,
                                onGet=self.onGet)
            else:
                return Selector(self.rootdir, self.params,
                                curdir=self.curdir,
                                filter=self.filter & expr,
                                onGet=self.onGet)
        else:
            return Selector(self.rootdir, self.params,
                            curdir=self.curdir,
                            filter=expr,
                            onGet=self.onGet)

    def or_(self):
        return Selector(self.rootdir, self.params,
                        curdir=self.curdir,
                        filter=self.filter,
                        next_is_or=True,
                        onGet=self.onGet)

    def _valid_params(self):
        ids = [get_id(fname) for fname in os.listdir( self.curdir )]
        valids = filter(lambda p: (p['id'] in ids) and (self.filter.eval(p)), self.params)
        return valids

    def size(self):
        v = self._valid_params()
        return len(v)

    def get(self, onGet=None):
        out = []
        # for p in self.params:
        #     fname = os.path.join(self.curdir, ('%d-' % p['id']) + os.path.split(self.curdir)[1])
        #     print fname
        #     if os.path.exists(fname) and self.filter.eval(p):
        #         if onGet:
        #             out.append( onGet(open(fname)) )
        #         else:
        #             out.append( self.onGet(open(fname)) )

        valids = self._valid_params()
        pbar = ProgressBar(num_iter=len(valids))

        if not onGet:
            onGet = self.onGet
        for i, v in enumerate(valids):
            fname = os.path.join(self.curdir, ('%d-' % v['id']) + os.path.split(self.curdir)[1])
            out.append( onGet(open(fname)) )
            pbar.update(i)

        return out



class MethodMissing(object):
    def __getattr__(self, name):
        try:
            return self.__getattribute__(name)
        except AttributeError:
            def method(*args, **kw):
                return self.method_missing(name, *args, **kw)
            return method

    def method_missing(self, name, *args, **kw):
        raise AttributeError("%r object has no attribute %r" %
                             (self.__class__, name))

class ValMissing(object):
    def __getattr__(self, name):
        try:
            return self.__getattribute__(name)
        except AttributeError:
            return self.val_missing(name)

    def val_missing(self, name):
        raise AttributeError("%r object has no attribute %r" %
                             (self.__class__, name))




import os
from setuptools import find_packages
from setuptools import setup
from gq import __version__

f = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
readme = f.read()
f.close()

setup(
    name='gq',
    version=__version__,
    description='a query-like selector for maf data',
    long_description=readme,
    author='Yoshihiro Nagano',
    author_email='y.nagano.92@gmail.com',
    licence='MIT License',
    url='http://github.com/ganow/gq/',
    install_requires = open('requirements.txt').read().splitlines(),
    packages=['gq'],
    classifiers = [
        'Programming Language :: Python :: 2 :: Only',
        'License :: OSI Approved :: MIT License'
    ]
)



from expr import Expression
import pytest

class TestExpr(object):

    def test_type(self, expr):
        assert isinstance(expr, Expression)

    def test_return_val(self, expr):
        assert str(expr) == "((((a >= 10) and (a < 20)) or (b == hoge)) and (c <= 30))"

if __name__ == '__main__':
    pytest.main()
